﻿using System;
using System.Collections.ObjectModel;
using System.Drawing;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;

namespace Post10399
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private string text;

        private void Form1_Load(object sender, EventArgs e)
        {
            winformsMap1.MapUnit = GeographyUnit.DecimalDegree;
            winformsMap1.CurrentExtent = new RectangleShape(-80, 80, 80, -80);
            winformsMap1.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.SimpleColors.White);//.FromArgb(255, 198, 255, 255));

            ShapeFileFeatureLayer fileLayer = new ShapeFileFeatureLayer(@"..\..\Data\Countries02.shp");
            fileLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = AreaStyles.CreateSimpleAreaStyle(GeoColor.StandardColors.Gray, GeoColor.SimpleColors.Black, 3);
            fileLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            InMemoryFeatureLayer cityLayer = new InMemoryFeatureLayer();
            cityLayer.Open();
            cityLayer.Columns.Add(new FeatureSourceColumn("CNTRY_NAME"));
            cityLayer.InternalFeatures.Add(new Feature(new PointShape(-95, 39), new string[] { "CNTRY_NAME:United States" }));
            cityLayer.InternalFeatures.Add(new Feature(new PointShape(104.194255828857, 35.8613128662109), new string[] { "CNTRY_NAME:China" }));
            cityLayer.InternalFeatures.Add(new Feature(new PointShape(50, 30), new string[] { "CNTRY_NAME:Unknown" }));


            PointStyle usaStyle = new PointStyle(new GeoImage(@"..\..\Data\Image.png"));
            PointStyle chinaStyle = new PointStyle(new GeoImage(@"..\..\Data\Image2.png"));
            PointStyle unknownStyle = new PointStyle(new GeoImage(@"..\..\Data\Unknown.png"));
            ValueStyle valueStyle = new ValueStyle();
            valueStyle.ColumnName = "CNTRY_NAME";
            valueStyle.ValueItems.Add(new ValueItem("United States", usaStyle));
            valueStyle.ValueItems.Add(new ValueItem("China", chinaStyle));
            valueStyle.ValueItems.Add(new ValueItem("Unknown", unknownStyle));

            cityLayer.ZoomLevelSet.ZoomLevel01.CustomStyles.Add(valueStyle);
            cityLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            cityLayer.Close();

            LayerOverlay overlay = new LayerOverlay();
            overlay.Layers.Add("WorldFileLayer", fileLayer);
            overlay.Layers.Add("CityLayer", cityLayer);

            winformsMap1.Overlays.Add("mapOverlay", overlay);

            winformsMap1.Refresh();
        }

        private void winformsMap1_MouseMove(object sender, MouseEventArgs e)
        {
            statusStrip1.Items[0].Text = "X:" + e.X + " Y:" + e.Y;
            PointShape pointShape = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, new ScreenPointF(e.X, e.Y), winformsMap1.Width, winformsMap1.Height);
            statusStrip1.Items[0].Text = "(world) X:" + Math.Round(pointShape.X, 4) + " Y:" + Math.Round(pointShape.Y, 4);
        }

        private void winformsMap1_MouseDown(object sender, MouseEventArgs e)
        {
            InMemoryFeatureLayer cityLayer = ((LayerOverlay)winformsMap1.Overlays["mapOverlay"]).Layers["CityLayer"] as InMemoryFeatureLayer;
            if (cityLayer != null)
            {
                cityLayer.Open();
                ValueStyle valueStyle = cityLayer.ZoomLevelSet.ZoomLevel01.CustomStyles[0] as ValueStyle;

                PointShape clickedPoint = ExtentHelper.ToWorldCoordinate(winformsMap1.CurrentExtent, e.X, e.Y, winformsMap1.Width, winformsMap1.Height);
                Collection<Feature> clickedFeatures = cityLayer.QueryTools.GetFeaturesNearestTo(clickedPoint, GeographyUnit.DecimalDegree, 1, ReturningColumnsType.AllColumns);
                cityLayer.Close();

                if (clickedFeatures.Count > 0)
                {
                    string cityName = clickedFeatures[0].ColumnValues["CNTRY_NAME"];
                    GeoImage featureImage = new GeoImage(0, 0);
                    foreach (var item in valueStyle.ValueItems)
                    {
                        if (item.Value == cityName)
                        {
                            featureImage = item.DefaultPointStyle.Image;
                        }
                    }

                    ScreenPointF screenPointF = ExtentHelper.ToScreenCoordinate(winformsMap1.CurrentExtent, clickedFeatures[0], winformsMap1.Width, winformsMap1.Height);
                    RectangleF rectangleF = new RectangleF(screenPointF.X - (featureImage.GetWidth() / 2), screenPointF.Y - (featureImage.GetHeight() / 2),
                                                           featureImage.GetWidth(), featureImage.GetHeight());

                    bool IsInside = rectangleF.Contains(new PointF(e.X, e.Y));

                    if (IsInside == true)
                    {
                        text = clickedFeatures[0].ColumnValues["CNTRY_NAME"];
                    }
                }
            }
        }

        private void winformsMap1_MouseUp(object sender, MouseEventArgs e)
        {
            if (!string.IsNullOrEmpty(text))
            {
                MessageBox.Show(text, "Country");
                text = string.Empty;
            }
        }
    }
}
